pub enum Error {
Deadlock,
NotBorn(NotBorn),
NotIdle(NotIdle),
NotBusy(NotBusy),
NotNext(NotNext),
NotDone(NotDone),
NotGone(NotGone),
NotInit(NotInit),
NotTerm(NotTerm),
Causality,
}Expand description
Enumeration of errors for the functions and structures contained in this module of the simulation library.
Variants
Deadlock
The simulation stopped when the simulator detected a deadlock.
NotBorn(NotBorn)
Continuation is already bound.
NotIdle(NotIdle)
Continuation is not idle.
NotBusy(NotBusy)
Continuation is not active.
NotNext(NotNext)
Continuation is not scheduled.
NotDone(NotDone)
Continuation hasn’t completed yet.
NotGone(NotGone)
Continuation hasn’t been released yet.
NotInit(NotInit)
Continuation hasn’t been bound yet.
NotTerm(NotTerm)
Continuation hasn’t terminated.
Causality
Signals an attempt to schedule a continuation at an earlier model-time.