StateMachine

Struct StateMachine 

pub struct StateMachine<'b, T: ?Sized> { /* private fields */ }
Wrapper type for custom state machines generated through the fsm macro.

Implementations

Source

impl<T> StateMachine<'static, T>

pub const fn new(inner: T) -> Self

Creates a new state machine using the state T.

Source

impl<'b, T: FSM> StateMachine<'b, T>

pub fn token(&self, once: Ephemeral<'b>) -> T::Token<'b>

Returns an enumeration containing token witnesses for the current state.

pub fn erased(&self) -> T::Erased

Returns a type-erased version of the current state, i.e. an enumeration of just the states, without additional payload.

pub fn label(&self) -> &'static str

Returns the name of the current state as a string.

pub fn transition<U, V>(&self, curr: U, data: T::Data) -> V
where T: Transition<'b, U, V>,

Performs a state transition given the current state and the payload of the follow-up state, returning a token witness for the next state.

This method will panic if the transition would invalidate other token witnesses. See Self::try_transition for a non-panicking variant.

pub fn try_transition<U, V>( &self, curr: U, data: T::Data, ) -> Result<V, TransitionError<U>>
where T: Transition<'b, U, V>,

Attempts to perform a state transition given the current state and the payload of the follow-up state, returning a token witness for the next state if successful and the previous token witness on error.

It is not possible to transition if other token witnesses for the same state machine would be invalidated by the transition. If you simply want to update the data associated with a state without changing it, consider using Self::update which doesn’t have this restriction.

pub fn update<U>(&self, curr: U, data: T::Data) -> U
where T: Transition<'b, U, U>,

Updates the internal data of a state without changing it.

This is allowed, even if other token witnesses exist as it cannot invalidate them and will only panic if the state cannot be modified due to being borrowed already.

pub fn borrow(&self) -> Ref<'_, T>

Borrows the inner state.