Simulator

Struct Simulator 

pub struct Simulator<C>(/* private fields */);
The principal driver of a simulation run which can be initialized using a configuration.

Implementations

Source

impl<C> Simulator<C>

pub const fn new(config: C) -> Self

Creates a new simulator from a configuration.

pub fn into_inner(self) -> C

Converts the simulator back into a configuration.

pub fn inner(&self) -> &C

Grants shared access to the inner configuration.

pub fn inner_mut(&mut self) -> &mut C

Grants mutable access to the inner configuration.

Source

impl<C: Config> Simulator<C>

pub fn run<F, R>(&self, main: F) -> Result<R, Deadlock<C::Time>>
where F: AsyncFnOnce(&Sim<C>) -> R,

Executes the simulation model provided in the closure and returns its result in the Ok-variant of a Result.

The Err-variant is reserved for errors that occur during the event-loop. Only a Deadlock can be reported.

pub fn with_result<F, T, E>(&self, main: F) -> Result<T, E>
where F: AsyncFnOnce(&Sim<C>) -> Result<T, E>, E: From<Deadlock<C::Time>>,

Executes the simulation model provided in the closure that has to return a Result.

The result of a simulation run can also be a Deadlock if the scheduler detects that it is not possible to schedule any more continuations after some model-time has passed.