pub struct Init<'brand>(/* private fields */);Expand description
Meta-state for all non-Born states.
Implementations
Sourceimpl<'brand> Init<'brand>
impl<'brand> Init<'brand>
Sourcepub const unsafe fn new(once: Ephemeral<'brand>) -> Self
pub const unsafe fn new(once: Ephemeral<'brand>) -> Self
Creates a new token of this type with an invariant lifetime 'brand.
Safety
Since tokens of this type are used to provide security guarantees, creating new ones should only be done if it is absolutely certain that the property symbolized by the token is actually upheld by the equally branded object it applies to.